@@ -29,6 +29,7 @@ extern	double	output_expires_after;
 	void	set_lastlog_mask 		(void *);
 	void	set_lastlog_size 		(void *);
 	void	set_notify_mask 		(void *);
+	int		recount_window_lastlog	(struct WindowStru *);
 	void	trim_lastlog			(struct WindowStru *);
 	void	set_current_window_mask 	(void *);
 	intmax_t add_to_lastlog 	(struct WindowStru *, const char *);
@@ -39,13 +40,10 @@ extern	double	output_expires_after;
 	void	reconstitute_scrollback		(struct WindowStru *);
 	int	do_expire_lastlog_entries	(void *);
 
-	void	move_all_lastlog		(unsigned, unsigned);
-	void	move_lastlog_item_by_string	(unsigned, unsigned, Char *);
-	void	move_lastlog_item_by_target	(unsigned, unsigned, Char *);
-	void	move_lastlog_item_by_level	(unsigned, unsigned, Mask *);
-	void	move_lastlog_item_by_regex	(unsigned, unsigned, Char *);
-
-	void    lastlog_swap_winrefs		(unsigned, unsigned);
-	void    lastlog_merge_winrefs		(unsigned, unsigned);
+	void	move_all_lastlog		(struct WindowStru *, struct WindowStru *);
+	void	move_lastlog_item_by_string	(struct WindowStru *, struct WindowStru *, Char *);
+	void	move_lastlog_item_by_target	(struct WindowStru *, struct WindowStru *, Char *);
+	void	move_lastlog_item_by_level	(struct WindowStru *, struct WindowStru *, Mask *);
+	void	move_lastlog_item_by_regex	(struct WindowStru *, struct WindowStru *, Char *);
 
 #endif /* __lastlog_h_ */
