@@ -1,4 +1,4 @@
-/* $EPIC: commands.c,v 1.200 2012/06/26 12:28:06 jnelson Exp $ */
+/* $EPIC: commands.c,v 1.208 2012/11/27 22:52:23 jnelson Exp $ */
 /*
  * commands.c -- Stuff needed to execute commands in ircII.
  *		 Includes the bulk of the built in commands for ircII.
@@ -798,7 +798,10 @@ BUILT_IN_COMMAND(e_topic)
 	const char *channel = get_echannel_by_refnum(0);
 	const char *arg;
 
-	if (args && *args == '-')
+	if (!args)
+		return;
+
+	if (*args == '-')
 		clear_topic = 1, args++;
 
 	args_copy = LOCAL_COPY(args);
@@ -947,11 +950,8 @@ BUILT_IN_COMMAND(xechocmd)
 
 		case 'V':	/* VISUAL (output to a visible window) */
 		{
-			/* There is always at least one visible window! */
-			Window *win = NULL;
-
 			/* Chew up the argument. */
-			flag_arg = next_arg(args, &args);
+			/*flag_arg =*/ next_arg(args, &args);
 
 			if (current_window->screen)
 			    to_window_refnum = current_window->refnum;
@@ -961,6 +961,8 @@ BUILT_IN_COMMAND(xechocmd)
 							current_window->refnum;
 			else
 			{
+			    /* There is always at least one visible window! */
+			    Window *win = NULL;
 			    while ((traverse_all_windows(&win)))
 			    {
 				if (win->screen)
@@ -1001,7 +1003,7 @@ BUILT_IN_COMMAND(xechocmd)
 
 		case 'E':
 		{
-			double	timeout;
+			double	timeout = 0;
 
 			next_arg(args, &args);
 			if (!(flag_arg = next_arg(args, &args)))
@@ -1009,9 +1011,11 @@ BUILT_IN_COMMAND(xechocmd)
 
 			/* Outputting here is bad.  So just ignore it */
 			if (is_number(flag_arg))
+			{
 				timeout = atof(flag_arg);
+				output_expires_after = timeout;
+			}
 
-			output_expires_after = timeout;
 			break;
 		}
 		case 'A':	/* ALL (output to all windows) */
@@ -1611,9 +1615,9 @@ BUILT_IN_COMMAND(subpackagecmd)
 	malloc_strcat(&load_level[load_depth].package, args);
 }
 
-static void	loader_which (struct epic_loadfile *elf, const char *filename, const char *args, struct load_info *);
-static void	loader_std (struct epic_loadfile *elf, const char *filename, const char *args, struct load_info *);
-static void	loader_pf  (struct epic_loadfile *elf, const char *filename, const char *args, struct load_info *);
+static void	loader_which (const char *file_contents, off_t file_contents_size, const char *filename, const char *args, struct load_info *);
+static void	loader_std (const char *file_contents, off_t file_contents_size, const char *filename, const char *args, struct load_info *);
+static void	loader_pf  (const char *file_contents, off_t file_contents_size, const char *filename, const char *args, struct load_info *);
 
 /*
  * load: the /LOAD command.  Reads the named file, parsing each line as
@@ -1630,7 +1634,10 @@ BUILT_IN_COMMAND(load)
         struct epic_loadfile * elf;
 	int	display;
 	int	do_one_more = 0;
-	void	(*loader) (struct epic_loadfile *, const char *, const char *, struct load_info *);
+	void	(*loader) (const char *, off_t, const char *, const char *, struct load_info *);
+	char *	file_contents = NULL;
+	off_t	file_contents_size = 0;
+	char *	declared_encoding = NULL;
 
         int             idx;
         int             err;
@@ -1699,6 +1706,11 @@ BUILT_IN_COMMAND(load)
 		do_one_more = 1;
 		continue;		/* Pick up the filename */
 	    }
+	    else if (my_strnicmp(filename, "-encoding", strlen(filename)) == 0)
+	    {
+		declared_encoding = next_arg(args, &args);
+		continue;
+	    }
 	    else
 		sargs = NULL;
 
@@ -1715,9 +1727,28 @@ BUILT_IN_COMMAND(load)
 	     */
 	    expanded = malloc_strdup(filename);
 
-            if (!(elf = uzfopen(&expanded, use_path, 1, &load_level[load_depth].sb)))
+	    /* Read the file into a string */
+            if (!(elf = uzfopen(&expanded, use_path, 1, 
+				&load_level[load_depth].sb)))
                 continue;
 
+	    if (slurp_elf_file(elf, &file_contents, &file_contents_size) > 0)
+	    {
+		if (declared_encoding)
+		{
+		    /* The 2nd arg should become /SET DEFAULT_SCRIPT_ENCODING */
+		    recode_with_iconv(declared_encoding, NULL, 
+				&file_contents, &file_contents_size);
+	        }
+	    }
+	    epic_fclose(elf);
+	    new_free(&elf);
+
+	    /* If no file resulted, then we're done. */
+	    if (!file_contents || !*file_contents)
+		continue;
+
+	    /* Now process the file */
             load_level[load_depth].filename = expanded;
 	    load_level[load_depth].line = 1;
 	    if (load_depth > 0 && load_level[load_depth - 1].package)
@@ -1725,14 +1756,14 @@ BUILT_IN_COMMAND(load)
 				load_level[load_depth-1].package);
 
 	    will_catch_return_exceptions++;
-	    loader(elf, expanded, sargs, &load_level[load_depth]);
+	    loader(file_contents, file_contents_size, expanded, 
+			sargs, &load_level[load_depth]);
 	    will_catch_return_exceptions--;
 	    return_exception = 0;
 
 	    new_free(&load_level[load_depth].filename);
 	    new_free(&load_level[load_depth].package);
-            epic_fclose(elf);
-            new_free(&elf);
+	    new_free(&file_contents);
 	}
 
 	/*
@@ -1747,14 +1778,14 @@ BUILT_IN_COMMAND(load)
 }
 
 /* The "WHICH" loader */
-static void	loader_which (struct epic_loadfile *elf, const char *filename, const char *subargs, struct load_info *loadinfo)
+static void	loader_which (const char *file_contents, off_t file_contents_size, const char *filename, const char *subargs, struct load_info *loadinfo)
 {
 	loadinfo->loader = "which";
 	yell("%s", filename);
 }
 
 /* The "Standard" (legacy) loader */
-static void	loader_std (struct epic_loadfile *elf, const char *filename, const char *subargs, struct load_info *loadinfo)
+static void	loader_std (const char *file_contents, off_t file_contents_size, const char *filename, const char *subargs, struct load_info *loadinfo)
 {
 	int	in_comment, comment_line, no_semicolon;
 	int	paste_level, paste_line;
@@ -1777,7 +1808,7 @@ static void	loader_std (struct epic_loadfile *elf, const char *filename, const c
 	    int     len;
 	    char    *ptr;
 
-            if (!epic_fgets(buffer, MAX_LINE_SIZE, elf))
+            if (!string_fgets(buffer, MAX_LINE_SIZE, &file_contents, &file_contents_size))
                     break;
 
             if (loadinfo->line == 1 && loadinfo->sb.st_mode & 0111 &&
@@ -1821,7 +1852,7 @@ static void	loader_std (struct epic_loadfile *elf, const char *filename, const c
 			(len >= 2 && start[len-2] == '\\') &&
 			(len < 3 || start[len-3] != '\\') && 
 		        (len < MAX_LINE_SIZE) && 
-                        (epic_fgets(&(start[len-2]), MAX_LINE_SIZE - len, elf)))
+                        (string_fgets(&(start[len-2]), MAX_LINE_SIZE - len, &file_contents, &file_contents_size)))
 	    {
 		len = strlen(start);
 		loadinfo->line++;
@@ -2083,7 +2114,7 @@ static void	loader_std (struct epic_loadfile *elf, const char *filename, const c
  * and is "signficantly" faster than the standard loader.  This can be a 
  * big win for large scripts.
  */
-static void	loader_pf (struct epic_loadfile *elf, const char *filename, const char *subargs, struct load_info *loadinfo)
+static void	loader_pf (const char *file_contents, off_t file_contents_size, const char *filename, const char *subargs, struct load_info *loadinfo)
 {
 	char *	buffer;
 	int	bufsize, pos;
@@ -2098,7 +2129,7 @@ static void	loader_pf (struct epic_loadfile *elf, const char *filename, const ch
 	comment = 0;
 	shebang = 0;
 
-	this_char = epic_fgetc(elf);
+	this_char = string_fgetc(&file_contents, &file_contents_size);
 
 	/* 
 	 * If the file is +x, and starts with #!, it's ok.
@@ -2107,9 +2138,9 @@ static void	loader_pf (struct epic_loadfile *elf, const char *filename, const ch
 	if (loadinfo->sb.st_mode & 0111)
 	{
 	    /* Special code to support #! scripts. */
-	    if (this_char == '#' && !epic_feof(elf))
+	    if (this_char == '#' && !string_feof(file_contents, file_contents_size))
 	    {
-		this_char = epic_fgetc(elf);
+		this_char = string_fgetc(&file_contents, &file_contents_size);
 		if (this_char == '!')
 		   shebang = 1;
 	    }
@@ -2125,7 +2156,7 @@ static void	loader_pf (struct epic_loadfile *elf, const char *filename, const ch
 	    comment = 1;
 	}
 
-	while (!epic_feof(elf))
+	while (!string_feof(file_contents, file_contents_size))
 	{
 	    do
 	    {
@@ -2167,7 +2198,7 @@ static void	loader_pf (struct epic_loadfile *elf, const char *filename, const ch
 		new_realloc((void **)&buffer, bufsize);
 	    }
 
-	    this_char = epic_fgetc(elf);
+	    this_char = string_fgetc(&file_contents, &file_contents_size);
 	}
 
 	buffer[pos] = 0;
