@@ -1,4 +1,4 @@
-/* $EPIC: if.c,v 1.26 2005/06/04 04:15:27 jnelson Exp $ */
+/* $EPIC: if.c,v 1.45 2012/11/25 05:56:28 jnelson Exp $ */
 /*
  * if.c: the IF, WHILE, FOREACH, DO, FE, FEC, and FOR commands for IRCII 
  *
@@ -35,18 +35,22 @@
 
 #include "irc.h"
 #include "alias.h"
-#include "if.h"
+#include "ifcmd.h"
 #include "ircaux.h"
 #include "output.h"
 #include "parse.h"
 #include "vars.h"
 #include "commands.h"
 #include "window.h"
-
+#include "reg.h"
 
 /*
  * next_expr finds the next expression delimited by brackets. The type
  * of bracket expected is passed as a parameter. Returns NULL on error.
+ *
+ * YOU MUST CHECK FOR AND PROPERLY HANDLE A NULL ON RETURN -- 
+ * PASSING NULL TO parse_line() or parse_arglist() OR SIMILAR FUNCTIONS
+ * WILL RETURN IN A PANIC.  DO NOT IGNORE THIS!
  */
 static __inline__
 char *	my_next_expr (char **args, char type, int whine, int wantchar)
@@ -67,7 +71,9 @@ char *	my_next_expr (char **args, char type, int whine, int wantchar)
 	if ((span = MatchingBracket(expr_start + 1, type, 
 					(type == '(') ? ')' : '}')) < 0)
 	{
-		say("Unmatched '%c'", type);
+		if (whine)
+			say("Unmatched '%c' around [%-.20s]", 
+				type, expr_start + 1);
 		return NULL;
 	}
 	else
@@ -98,16 +104,31 @@ char *	my_next_expr (char **args, char type, int whine, int wantchar)
 	return expr_start;
 }
 
+/*
+ * YOU MUST CHECK FOR AND PROPERLY HANDLE A NULL ON RETURN -- 
+ * PASSING NULL TO parse_line() or parse_arglist() OR SIMILAR FUNCTIONS
+ * WILL RETURN IN A PANIC.  DO NOT IGNORE THIS!
+ */
 char *	next_expr_with_type (char **args, char type)
 {
 	return my_next_expr (args, type, 0, 1);
 }
 
+/*
+ * YOU MUST CHECK FOR AND PROPERLY HANDLE A NULL ON RETURN -- 
+ * PASSING NULL TO parse_line() or parse_arglist() OR SIMILAR FUNCTIONS
+ * WILL RETURN IN A PANIC.  DO NOT IGNORE THIS!
+ */
 char *	next_expr_failok (char **args, char type)
 {
 	return my_next_expr (args, type, 0, 0);
 }
 
+/*
+ * YOU MUST CHECK FOR AND PROPERLY HANDLE A NULL ON RETURN -- 
+ * PASSING NULL TO parse_line() or parse_arglist() OR SIMILAR FUNCTIONS 
+ * WILL RETURN IN A PANIC.  DO NOT IGNORE THIS!
+ */
 char *	next_expr (char **args, char type)
 {
 	return my_next_expr (args, type, 1, 0);
@@ -136,7 +157,6 @@ BUILT_IN_COMMAND(ifcmd)
 	char *current_expr_val;
 	int result;
 	char *stuff = NULL;
-	int flag = 0;
 
 	unless_cmd = (*command == 'U');
 	if (!subargs)
@@ -153,10 +173,10 @@ BUILT_IN_COMMAND(ifcmd)
 		current_expr = next_expr(&args, '(');
 		if (!current_expr)
 		{
-			error("IF: Missing expression");
+			my_error("IF: Missing expression");
 			return;
 		}
-		current_expr_val = parse_inline(current_expr, subargs, &flag);
+		current_expr_val = parse_inline(current_expr, subargs);
 		if (get_int_var(DEBUG_VAR) & DEBUG_EXPANSIONS)
 			privileged_yell("%s expression expands to: (%s)", command, current_expr_val);
 
@@ -205,7 +225,7 @@ BUILT_IN_COMMAND(ifcmd)
 		}
 
 		if (stuff)
-			parse_line(NULL, stuff, subargs, 0, 0);
+			runcmds(stuff, subargs);
 
 		break;
 	}
@@ -215,7 +235,6 @@ BUILT_IN_COMMAND(docmd)
 {
 	char *body, *expr, *cmd, *ptr;
 	char *newexp = NULL;
-	int args_used = 0;
 	int result;
 
 	if (!subargs)
@@ -225,7 +244,7 @@ BUILT_IN_COMMAND(docmd)
 	{
 		if (!(body = next_expr(&args, '{')))
 		{
-			error("DO: unbalanced {");
+			my_error("DO: unbalanced {");
 			return;
 		}	
 		if (args && *args && (cmd = next_arg(args, &args)) && 
@@ -233,7 +252,7 @@ BUILT_IN_COMMAND(docmd)
 		{
 			if (!(expr = next_expr(&args, '(')))
 			{
-				error("DO: unbalanced (");
+				my_error("DO: unbalanced (");
 				return;
 			}
 
@@ -241,7 +260,7 @@ BUILT_IN_COMMAND(docmd)
 			will_catch_continue_exceptions++;
 			while (1)
 			{
-				parse_line(NULL, body, subargs, 0, 0);
+				runcmds(body, subargs);
 				if (break_exception)
 				{
 					break_exception = 0;
@@ -254,10 +273,12 @@ BUILT_IN_COMMAND(docmd)
 				}
 				if (return_exception)
 					break;
+				if (system_exception)
+					break;
 
 				/* Alas, too bad the malloc is neccesary */
 				malloc_strcpy(&newexp, expr);
-				ptr = parse_inline(newexp, subargs, &args_used);
+				ptr = parse_inline(newexp, subargs);
 				result = check_val(ptr);
 				new_free(&ptr);
 				if (!result)
@@ -269,10 +290,10 @@ BUILT_IN_COMMAND(docmd)
 			return;
 		}
 		/* falls through to here if its /do {...} */
-		parse_line(NULL, body, subargs, 0, 0);
+		runcmds(body, subargs);
 	}
 	/* falls through to here if it its /do ... */
-	parse_line(NULL, args, subargs, 0, 0);
+	runcmds(args, subargs);
 }
 
 BUILT_IN_COMMAND(whilecmd)
@@ -281,22 +302,30 @@ BUILT_IN_COMMAND(whilecmd)
 		*ptr,
 		*body = NULL,
 		*newexp = NULL;
-	int	args_used;	/* this isn't used here, but is passed
-				 * to expand_alias() */
 	int 	whileval = !strcmp(command, "WHILE");
 
+	if (!args)
+		return;
+
 	if (!subargs)
 		subargs = empty_string;
 
 	if (!(ptr = next_expr(&args, '(')))
 	{
-		error("WHILE: missing boolean expression");
+		my_error("WHILE: missing boolean expression");
 		return;
 	}
 	exp = LOCAL_COPY(ptr);
 
 	if (!(ptr = next_expr_failok(&args, '{')))
-		ptr = args;
+	{
+		if (!(ptr = args))
+		{
+			my_error("WHILE: Missing body");
+			return;
+		}
+	}
+
 	body = LOCAL_COPY(ptr);
 
 	will_catch_break_exceptions++;
@@ -305,13 +334,13 @@ BUILT_IN_COMMAND(whilecmd)
 	while (1)
 	{
 		strlcpy(newexp, exp, strlen(exp) + 1);
-		ptr = parse_inline(newexp, subargs, &args_used);
+		ptr = parse_inline(newexp, subargs);
 		if (check_val(ptr) != whileval)
 			break;
 
 		new_free(&ptr);
 
-		parse_line(NULL, body, subargs, 0, 0);
+		runcmds(body, subargs);
 		if (continue_exception)
 		{
 			continue_exception = 0;
@@ -324,6 +353,8 @@ BUILT_IN_COMMAND(whilecmd)
 		}
 		if (return_exception)
 			break;
+		if (system_exception)
+			break;
 	}
 	will_catch_break_exceptions--;
 	will_catch_continue_exceptions--;
@@ -340,9 +371,7 @@ BUILT_IN_COMMAND(foreach)
 	int	total;
 	int	i;
 	int	slen;
-	int	old_display;
 	int	list = VAR_ALIAS;
-	int	af;
 
 	if (!subargs)
 		subargs = empty_string;
@@ -350,21 +379,27 @@ BUILT_IN_COMMAND(foreach)
 	while (args && my_isspace(*args))
 		args++;
 
+	if (!args || !*args)
+	{
+		my_error("Usage: FOREACH structure varname {block}");
+		return;
+	}
+
 	if (*args == '-')
 		args++, list = COMMAND_ALIAS;
 
 	if (!(ptr = new_next_arg(args, &args)))
 	{
-		error("FOREACH: missing structure expression");
+		my_error("FOREACH: missing structure expression");
 		return;
 	}
 
-	struc = upper(remove_brackets(ptr, subargs, &af));
+	struc = upper(remove_brackets(ptr, subargs));
 
 	if (!(var = next_arg(args, &args)))
 	{
 		new_free(&struc);
-		error("FOREACH: missing variable");
+		my_error("FOREACH: missing variable");
 		return;
 	}
 	while (my_isspace(*args))
@@ -373,7 +408,7 @@ BUILT_IN_COMMAND(foreach)
 	if (!(body = next_expr(&args, '{')))
 	{
 		new_free(&struc);
-		error("FOREACH: missing statement");
+		my_error("FOREACH: missing statement");
 		return;
 	}
 
@@ -384,7 +419,6 @@ BUILT_IN_COMMAND(foreach)
 	}
 
 	slen = strlen(struc);
-	old_display = window_display;
 
 	will_catch_break_exceptions++;
 	will_catch_continue_exceptions++;
@@ -393,7 +427,7 @@ BUILT_IN_COMMAND(foreach)
 		add_local_alias(var, sublist[i] + slen + 1, 0);
 		new_free(&sublist[i]);
 
-		parse_line(NULL, body, subargs, 0, 0);
+		runcmds(body, subargs);
 	
 		if (continue_exception)
 		{
@@ -407,6 +441,8 @@ BUILT_IN_COMMAND(foreach)
 		}
 		if (return_exception)
 			break;
+		if (system_exception)
+			break;
 	}
 	while (++i < total)
 		new_free(&sublist[i]);
@@ -418,8 +454,7 @@ BUILT_IN_COMMAND(foreach)
 }
 
 /*
- * FE:  Written by Jeremy Nelson (jnelson@acronet.net)
- * When you need to iteratively loop rather than recursively loop.
+ * FE:  When you need to iteratively loop rather than recursively loop.
  */
 BUILT_IN_COMMAND(fe)
 {
@@ -432,19 +467,13 @@ BUILT_IN_COMMAND(fe)
 		*todo = NULL,
 		fec_buffer[2];
 	unsigned	ind, x, y;
-	int     args_flag;
 	int     old_display;
 	int	doing_fe = !strcmp(command, "FE");
 	char	*mapvar = NULL;
-	char	*mapsep = doing_fe ? space : empty_string;
+	const char	*mapsep = doing_fe ? space : empty_string;
 	char	*map = NULL;
 	size_t	mapclue = 0;
 
-#if 0
-	for (x = 0; x <= (sizeof(var) / sizeof(*var)); var[x++] = NULL)
-		;
-#endif
-
 	if (!subargs)
 		subargs = empty_string;
 
@@ -452,9 +481,9 @@ BUILT_IN_COMMAND(fe)
 		mapvar = next_arg(args, &args);
 		templist = get_variable(mapvar);
 	} else if ((list = next_expr(&args, '('))) {
-		templist = expand_alias(list, subargs, &args_flag, NULL);
+		templist = expand_alias(list, subargs);
 	} else {
-		error("%s: Missing List for /%s", command, command);
+		my_error("%s: Missing List for /%s", command, command);
 		return;
 	}    
 
@@ -465,12 +494,12 @@ BUILT_IN_COMMAND(fe)
 
 	vars = args;
 	if (!(args = strchr(args, '{'))) {
-		error("%s: Missing commands", command);
+		my_error("%s: Missing commands", command);
 		new_free(&templist);
 		return;
 	}
 	if (vars == args) {
-		error("%s: You did not specify any variables", command);
+		my_error("%s: You did not specify any variables", command);
 		new_free(&templist);
 		return;
 	}
@@ -482,7 +511,7 @@ BUILT_IN_COMMAND(fe)
 	{
 		if (ind > (sizeof(var) / sizeof(*var)))
 		{
-			error("%s: Too many variables", command);
+			my_error("%s: Too many variables", command);
 			new_free(&templist);
 			return;
 		}
@@ -491,7 +520,7 @@ BUILT_IN_COMMAND(fe)
 
 	if (!(todo = next_expr(&args, '{')))
 	{
-		error("%s: Missing }", command);
+		my_error("%s: Missing }", command);
 		new_free(&templist);
 		return;
 	}
@@ -510,18 +539,18 @@ BUILT_IN_COMMAND(fe)
 		for ( y = 0 ; y < ind ; y++ )
 		{
 			if (doing_fe)
-				word = new_next_arg(templist, &templist);
+				word = next_func_arg(templist, &templist);
 			else
 				word[0] = *templist ? *templist++ : 0;
 
 			/* Something is really hosed if this happens */
 			if (!word)
-				word = empty_string;
+				word = endstr(templist);
 
 			add_local_alias(var[y], word, 0);
 		}
 		x += ind;
-		parse_line(NULL, todo, subargs, 0, 0);
+		runcmds(todo, subargs);
 
 		if (mapvar)
 			for ( y = 0 ; y < ind ; y++ ) {
@@ -542,6 +571,8 @@ BUILT_IN_COMMAND(fe)
 		}
 		if (return_exception)
 			break;
+		if (system_exception)
+			break;
 	}
 	will_catch_break_exceptions--;
 	will_catch_continue_exceptions--;
@@ -567,7 +598,7 @@ static void	for_next_cmd (int argc, char **argv, const char *subargs)
 	if ((my_stricmp(argv[1], "from") && my_stricmp(argv[1], "=")) ||
 		(argc != 6 && argc != 8))
 	{
-		error("Usage: /FOR var FROM start TO end {commands}");
+		my_error("Usage: /FOR var FROM start TO end {commands}");
 		return;
 	}
 
@@ -593,7 +624,7 @@ static void	for_next_cmd (int argc, char **argv, const char *subargs)
 	{
 		snprintf(istr, 255, "%d", i);
 		add_local_alias(var, istr, 0);
-		parse_line(NULL, cmds, subargs, 0, 0);
+		runcmds(cmds, subargs);
 
 		if (break_exception)
 		{
@@ -604,6 +635,8 @@ static void	for_next_cmd (int argc, char **argv, const char *subargs)
 			continue_exception = 0;	/* Dont continue here! */
 		if (return_exception)
 			break;
+		if (system_exception)
+			break;
 	}
 	will_catch_break_exceptions--;
 	will_catch_continue_exceptions--;
@@ -613,13 +646,12 @@ static void	for_fe_cmd (int argc, char **argv, const char *subargs)
 {
 	char 	*var, *list, *cmds;
 	char	*next, *real_list, *x;
-	int	args_flag = 0;
 
 	if (!subargs)
 		subargs = empty_string;
 
 	if ((my_stricmp(argv[1], "in")) || (argc != 4)) {
-		error("Usage: /FOR var IN (list) {commands}");
+		my_error("Usage: /FOR var IN (list) {commands}");
 		return;
 	}
 
@@ -631,14 +663,14 @@ static void	for_fe_cmd (int argc, char **argv, const char *subargs)
 		cmds++;
 	if (*list == '(')
 		list++;
-	x = real_list = expand_alias(list, subargs, &args_flag, NULL);
+	x = real_list = expand_alias(list, subargs);
 	will_catch_break_exceptions++;
 	will_catch_continue_exceptions++;
 	while (real_list && *real_list)
 	{
-		next = new_next_arg(real_list, &real_list);
+		next = next_func_arg(real_list, &real_list);
 		add_local_alias(var, next, 0);
-		parse_line(NULL, cmds, subargs, 0, 0);
+		runcmds(cmds, subargs);
 
 		if (break_exception) {
 			break_exception = 0;
@@ -648,6 +680,8 @@ static void	for_fe_cmd (int argc, char **argv, const char *subargs)
 			continue_exception = 0;	/* Dont continue here! */
 		if (return_exception)
 			break;
+		if (system_exception)
+			break;
 	}
 	will_catch_break_exceptions--;
 	will_catch_continue_exceptions--;
@@ -656,7 +690,7 @@ static void	for_fe_cmd (int argc, char **argv, const char *subargs)
 
 static void	for_pattern_cmd (int argc, char **argv, const char *subargs)
 {
-	error("/FOR var IN <pattern> {commands} reserved for future use");
+	my_error("/FOR var IN <pattern> {commands} reserved for future use");
 }
 
 static BUILT_IN_COMMAND(loopcmd)
@@ -670,20 +704,20 @@ static BUILT_IN_COMMAND(loopcmd)
 	argc = split_args(args, argv, 9);
 
 	if (argc < 2)
-		error("%s: syntax error", command);
+		my_error("%s: syntax error", command);
 	else if (!my_stricmp(argv[1], "from") || !my_stricmp(argv[1], "="))
 		for_next_cmd(argc, argv, subargs);
 	else if (!my_stricmp(argv[1], "in"))
 	{
 		if (argc < 4)
-			error("%s: syntax error", command);
+			my_error("%s: syntax error", command);
 		else if (*argv[2] == '(')
 			for_fe_cmd(argc, argv, subargs);
 		else
 			for_pattern_cmd(argc, argv, subargs);
 	}
 	else
-		error("%s: syntax error", command);
+		my_error("%s: syntax error", command);
 }
 
 /* 
@@ -705,7 +739,6 @@ BUILT_IN_COMMAND(forcmd)
 	char        *evaluation     = NULL;
 	char        *lameeval       = NULL;
 	char        *iteration      = NULL;
-	int         argsused        = 0;
 	char        *blah           = NULL;
 	char        *commands       = NULL;
 
@@ -724,7 +757,7 @@ BUILT_IN_COMMAND(forcmd)
 	/* Find the beginning of the second expression */
 	if (!(evaluation = strchr(commence, ',')))
 	{
-		error("FOR: no components!");
+		my_error("FOR: no components!");
 		return;
 	}
 	do 
@@ -734,7 +767,7 @@ BUILT_IN_COMMAND(forcmd)
 	/* Find the beginning of the third expression */
 	if (!(iteration = strchr(evaluation, ',')))
 	{
-		error("FOR: Only two components!");
+		my_error("FOR: Only two components!");
 		return;
 	}
 	do 
@@ -747,12 +780,12 @@ BUILT_IN_COMMAND(forcmd)
 
 	if (!(working = next_expr(&working, '{')))		/* } */
 	{
-		error("FOR: badly formed commands");
+		my_error("FOR: badly formed commands");
 		return;
 	}
 	commands = LOCAL_COPY(working);
 
-	parse_line(NULL, commence, subargs, 0, 0);
+	runcmds(commence, subargs);
 
 	will_catch_break_exceptions++;
 	will_catch_continue_exceptions++;
@@ -760,8 +793,7 @@ BUILT_IN_COMMAND(forcmd)
 	while (1)
 	{
 		strlcpy(lameeval, evaluation, strlen(evaluation) + 1);
-
-		blah = parse_inline(lameeval, subargs, &argsused);
+		blah = parse_inline(lameeval, subargs);
 		if (!check_val(blah))
 		{
 			new_free(&blah);
@@ -769,7 +801,7 @@ BUILT_IN_COMMAND(forcmd)
 		}
 
 		new_free(&blah);
-		parse_line(NULL, commands, subargs, 0, 0);
+		runcmds(commands, subargs);
 		if (break_exception)
 		{
 			break_exception = 0;
@@ -779,8 +811,10 @@ BUILT_IN_COMMAND(forcmd)
 			continue_exception = 0;	/* Dont continue here! */
 		if (return_exception)
 			break;
+		if (system_exception)
+			break;
 
-		parse_line(NULL, iteration, subargs, 0, 0);
+		runcmds(iteration, subargs);
 	}
 	will_catch_break_exceptions--;
 	will_catch_continue_exceptions--;
@@ -830,7 +864,6 @@ BUILT_IN_COMMAND(switchcmd)
 		*body, 
 		*header, 
 		*commands;
-	int 	af;
 
 	if (!subargs)
 		subargs = empty_string;
@@ -841,19 +874,13 @@ BUILT_IN_COMMAND(switchcmd)
 		return;
 	}
 
-	control = expand_alias(control, subargs, &af, NULL);
+	control = expand_alias(control, subargs);
 	if (get_int_var(DEBUG_VAR) & DEBUG_EXPANSIONS)
 		privileged_yell("%s expression expands to: (%s)", command, control);
 
 	if (!(body = next_expr(&args, '{')))
 		say("SWITCH: Execution body not found where expected");
 
-#if 0
-	/* Strip out the "" magic */
-	if (*body == '"' && body[strlen(body)-1] == '"')
-		body[strlen(body)-1] = 0, body++;
-#endif
-
 	while (body && *body)
 	{
 		int hooked = 0;
@@ -866,7 +893,7 @@ BUILT_IN_COMMAND(switchcmd)
 				new_free(&control);
 				return;
 			}
-			header = expand_alias(header, subargs, &af, NULL);
+			header = expand_alias(header, subargs);
 			if (get_int_var(DEBUG_VAR) & DEBUG_EXPANSIONS)
 				privileged_yell("%s expression expands to: (%s)", command, header);
 			if (wild_match(header, control))
@@ -885,7 +912,7 @@ BUILT_IN_COMMAND(switchcmd)
 		if (hooked)
 		{
 			will_catch_break_exceptions++;
-			parse_line(NULL, commands, subargs, 0, 0);
+			runcmds(commands, subargs);
 			if (break_exception)
 				break_exception = 0;
 			will_catch_break_exceptions--;
@@ -914,11 +941,14 @@ BUILT_IN_COMMAND(repeatcmd)
 	if (*args == '(')
 	{
 		char *		dumb_copy;
-		int 		argsused;
 
-		num_expr = next_expr(&args, '(');
+		if (!(num_expr = next_expr(&args, '(')))
+		{
+			say("REPEAT: Could not find expression");
+			return;
+		}
 		dumb_copy = LOCAL_COPY(num_expr);
-		tmp_val = parse_inline(dumb_copy, subargs, &argsused);
+		tmp_val = parse_inline(dumb_copy, subargs);
 	}
 	else
 	{
@@ -934,7 +964,7 @@ BUILT_IN_COMMAND(repeatcmd)
 
 	/* Probably want to catch break and continue here */
 	while (value--)
-		parse_line(NULL, args, subargs, 0, 0);
+		runcmds(args, subargs);
 
 	return;
 }
