@@ -1,4 +1,4 @@
-/* $EPIC: irc.c,v 1.1336 2012/08/05 01:28:03 jnelson Exp $ */
+/* $EPIC: irc.c,v 1.1352 2012/11/28 23:10:58 jnelson Exp $ */
 /*
  * ircII: a new irc client.  I like it.  I hope you will too!
  *
@@ -40,19 +40,19 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC5-1.1.4";
-const char useful_info[] = "epic5 1 1 4";
+const char irc_version[] = "EPIC5-1.1.5";
+const char useful_info[] = "epic5 1 1 5";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20120804";
+const char internal_version[] = "20121128";
 
 /*
  * In theory, this number is incremented for every commit.
  */
-const unsigned long	commit_id = 1667;
+const unsigned long	commit_id = 1683;
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -60,7 +60,7 @@ const unsigned long	commit_id = 1667;
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Adumbration";
+const char ridiculous_version_name[] = "Gallimaufry";
 
 #define __need_putchar_x__
 #include "status.h"
@@ -89,6 +89,7 @@ const char ridiculous_version_name[] = "Adumbration";
 #include "parse.h"
 #include "levels.h"
 #include "extlang.h"
+#include "files.h"
 #include <pwd.h>
 
 
@@ -834,11 +835,8 @@ static	void	parse_args (int argc, char **argv)
 	{
 		int argn;
 		for (argn = 0; argn < altargc; argn++)
-		{
 			new_free(&(altargv[altargc]));
-		}
 		new_free(&altargv);
-		altargc = 0;
 	}
 	
 	return;
@@ -984,8 +982,10 @@ static	int		level = 0,
 	/* Move the cursor back to the input line */
 	cursor_to_input();
 
-	/* Reclaim any alloca()ed space */
+#ifdef __GNUC__
+	/* GCC wants us to do this to reclaim any alloca()ed space */
 	alloca(0);
+#endif
 
 	/* Release this io() accounting level */
 	caller[level] = NULL;
@@ -1170,6 +1170,7 @@ void    load_ircrc (void)
 /*************************************************************************/
 int 	main (int argc, char *argv[])
 {
+	setlocale(LC_ALL, "");
 	setvbuf(stdout, NULL, _IOLBF, 1024);
 #ifdef SOCKS
 	SOCKSinit(argv[0]);
