@@ -1,4 +1,4 @@
-/* $EPIC: screen.c,v 1.145 2012/07/06 22:38:21 jnelson Exp $ */
+/* $EPIC: screen.c,v 1.149 2012/11/28 03:12:07 jnelson Exp $ */
 /*
  * screen.c
  *
@@ -218,7 +218,7 @@ static size_t	display_attributes (unsigned char *output, Attribute *old_a, Attri
 	output[4] = val4;
 	output[5] = 0;
 
-	old_a = a;
+	old_a = a;	/* XXX - This should be *old_a = *a, right? */
 	return 5;
 }
  
@@ -750,7 +750,6 @@ start_over:
 	    /*
 	     * Set up the arguments list
 	     */
-	    nargs = 0;
 	    args[0] = args[1] = args[2] = args[3] = 0;
 	    args[4] = args[5] = args[6] = args[7] = 0;
 	    args[8] = args[9] = 0;
@@ -1391,6 +1390,9 @@ unsigned char *	denormalize_string (const unsigned char *str)
 	 * The output string has a few extra chars on the end just 
 	 * in case you need to tack something else onto it.
 	 */
+	if (!str)
+		str = "<denormalize_string was called with NULL>";
+
 	maxpos = strlen(str);
 	output = (unsigned char *)new_malloc(maxpos + 192);
 	pos = 0;
@@ -2541,19 +2543,6 @@ void 	cursor_in_display (Window *w)
 		w->screen->cursor_window = w;
 }
 
-/*
- * is_cursor_in_display: returns true if the cursor is in one of the windows
- * (cursor_window is not null), false otherwise 
- */
-int 	is_cursor_in_display (Screen *screen)
-{
-	if (!screen && current_window->screen)
-		screen = current_window->screen;
-
-	return (screen->cursor_window ? 1 : 0);
-}
-
-
 /* * * * * * * SCREEN UDPATING AND RESIZING * * * * * * * * */
 /*
  * repaint_window_body: redraw the entire window's scrollable region
@@ -2742,8 +2731,6 @@ Window	*create_additional_screen (void)
 #else
         Window  	*win;
         Screen  	*oldscreen, *new_s;
-        char    	*displayvar,
-                	*termvar;
         int     	screen_type = ST_NOTHING;
 	ISA		local_sockaddr;
         ISA		new_socket;
@@ -2792,11 +2779,7 @@ Window	*create_additional_screen (void)
 	if (screen_type == ST_SCREEN)
 		say("Opening new screen...");
 	else if (screen_type == ST_XTERM)
-	{
-		displayvar = getenv("DISPLAY");
-		termvar = getenv("TERM");
 		say("Opening new window...");
-	}
 	else
 		panic(1, "Opening new wound");
 
@@ -2846,8 +2829,10 @@ Window	*create_additional_screen (void)
 			char geom[32];
 			int i;
 
-			setuid(getuid());
-			setgid(getgid());
+			if (setuid(getuid()))
+				_exit(0);
+			if (setgid(getgid()))
+				_exit(0);
 			setsid();
 
 			/*
@@ -3187,11 +3172,13 @@ static void 	do_screens (int fd)
 } 
 
 /*
- * This function accumulates bytes of user input encoded in 
- * /SET INPUT_TRANSLATION and converts them to (UCS32) code points and
- * calls edit_codepoint() once per code point.
+ * Each byte received by user input goes through this function.
+ * This function needs to decide what to do -- do we accumulate
+ * a UTF8 string, convert it to UCS32, and send it to edit_char()?
+ * or do we translate it based on /set translation and send that
+ * to edit_char()?
  */
-static	void translate_user_input (char byte)
+static	void	translate_user_input (char byte)
 {
 static	char		workbuf[32];
 static	size_t		workbuf_idx = 0;
@@ -3201,43 +3188,11 @@ static	size_t		workbuf_idx = 0;
 	edit_codepoint(codepoint);
 }
 
-
-/* * * * * * * * * INPUT PROMPTS * * * * * * * * * * */
-/* 
- * add_wait_prompt:  Given a prompt string, a function to call when
- * the prompt is entered.. some other data to pass to the function,
- * and the type of prompt..  either for a line, or a key, we add 
- * this to the prompt_list for the current screen..  and set the
- * input prompt accordingly.
- *
- * XXXX - maybe this belongs in input.c? =)
+/*
+ * This should be called once for each codepoint we receive.
+ * This means a utf8 string converted into UCS32.  We don't support
+ * that quite yet, so this is just a placeholder.
  */
-void 	add_wait_prompt (const char *prompt, void (*func)(char *, char *), const char *data, int type, int echo)
-{
-	WaitPrompt **AddLoc,
-		   *New;
-	Screen *	s;
-
-	if (current_window->screen)
-		s = current_window->screen;
-	else
-		s = main_screen;
-
-	New = (WaitPrompt *) new_malloc(sizeof(WaitPrompt));
-	New->prompt = malloc_strdup(prompt);
-	New->data = malloc_strdup(data);
-	New->type = type;
-	New->echo = echo;
-	New->func = func;
-	New->next = NULL;
-
-	for (AddLoc = &s->promptlist; *AddLoc; AddLoc = &(*AddLoc)->next)
-		/* nothing */;
-	*AddLoc = New;
-	if (AddLoc == &s->promptlist)
-		change_input_prompt(1);
-}
-
 void	edit_codepoint (u_32int_t key)
 {
 	unsigned char	uckey;
@@ -3327,3 +3282,40 @@ void    edit_char (unsigned char key)
         }
 }
 
+/* * * * * * * * * INPUT PROMPTS * * * * * * * * * * */
+/* 
+ * add_wait_prompt:  Given a prompt string, a function to call when
+ * the prompt is entered.. some other data to pass to the function,
+ * and the type of prompt..  either for a line, or a key, we add 
+ * this to the prompt_list for the current screen..  and set the
+ * input prompt accordingly.
+ *
+ * XXXX - maybe this belongs in input.c? =)
+ */
+void 	add_wait_prompt (const char *prompt, void (*func)(char *, char *), const char *data, int type, int echo)
+{
+	WaitPrompt **AddLoc,
+		   *New;
+	Screen *	s;
+
+	if (current_window->screen)
+		s = current_window->screen;
+	else
+		s = main_screen;
+
+	New = (WaitPrompt *) new_malloc(sizeof(WaitPrompt));
+	New->prompt = malloc_strdup(prompt);
+	New->data = malloc_strdup(data);
+	New->type = type;
+	New->echo = echo;
+	New->func = func;
+	New->next = NULL;
+
+	for (AddLoc = &s->promptlist; *AddLoc; AddLoc = &(*AddLoc)->next)
+		/* nothing */;
+	*AddLoc = New;
+	if (AddLoc == &s->promptlist)
+		change_input_prompt(1);
+}
+
+
