@@ -1,4 +1,4 @@
-/* $EPIC: who.c,v 1.66 2012/06/11 03:56:26 jnelson Exp $ */
+/* $EPIC: who.c,v 1.72 2012/11/28 03:12:07 jnelson Exp $ */
 /*
  * who.c -- The WHO queue.  The ISON queue.  The USERHOST queue.
  *
@@ -965,16 +965,20 @@ void	who_end (int refnum, const char *from, const char *comm, const char **ArgLi
 	while (new_w->piggyback && (new_w = new_w->next));
 	pop_message_from(l);
 
-	if (!*new_w->who_target)
+	if (new_w)
 	{
+	   if (!*new_w->who_target)
+	   {
 		WHO_DEBUG("Popping off for server [%d]", refnum);
 		who_queue_pop(refnum);
-	}
-	else
-	{
+	   }
+	   else
+	   {
 		WHO_DEBUG("NOT POPPING OFF FOR SERVER [%d] BECAUSE THE TOP OF THE WHO QUEUE HAS [%s] LEFT IN THE TARGET LIST", refnum, new_w->who_target);
 		yell("WHOEND: Caution -- not popping off who queue -- [%s] is left on target list", new_w->who_target);
+	   }
 	}
+	/* XXX - There should probably be an else here, to cover when new_w had a piggyback, but there was nothing to piggyback on */
 
 	new_free(&target);
 }
@@ -1019,7 +1023,7 @@ int	fake_who_end (int refnum, const char *from, const char *comm, const char *wh
 		if (strncmp(new_w->who_target, target, strlen(target)))
 			return 0;
 
-		who_target = target;
+		/*who_target = target; */
 	}
 
 	l = message_from(new_w->who_target, LEVEL_OTHER);
@@ -1230,7 +1234,7 @@ BUILT_IN_COMMAND(isoncmd)
 void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 {
 	IsonEntry 	*new_i;
-	char 		*next, *end, *p;
+	char 		*next = NULL, *end, *p;
 	char		*on_cmd = NULL, *offcmd = NULL, *endcmd = NULL;
 	int		sendnext = 0;
 
@@ -1245,14 +1249,14 @@ void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 
 		if (!my_stricmp(arg, "-d"))
 		{
-			ison_queue_list(from_server);
+			ison_queue_list(refnum);
 		}
 		if (!my_stricmp(arg, "-f"))
 		{
-			while (ison_queue_top(from_server))
-				ison_queue_pop(from_server);
-			while (ison_wait_top(from_server))
-				ison_wait_pop(from_server);
+			while (ison_queue_top(refnum))
+				ison_queue_pop(refnum);
+			while (ison_wait_top(refnum))
+				ison_wait_pop(refnum);
 		}
 		if (!my_stricmp(arg, "-s"))
 		{
@@ -1262,21 +1266,29 @@ void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 		{
 			sendnext++;
 		}
-		if (!my_stricmp(arg, "-e") && get_server(refnum)->ison_wait)
+		if (!my_stricmp(arg, "-e"))
 		{
-			return;
+			if (get_server(refnum))
+				if (get_server(refnum)->ison_wait)
+					return;
 		}
 		if (!my_stricmp(arg, "-len"))
 		{
 			if ((stuff = next_arg(args, &args)))
+			{
+			     if (get_server(refnum))
 				get_server(refnum)->ison_len = MAX(100, atol(stuff));
+			}
 			else
 				say("Need numeric argument for -LEN argument.");
 		}
 		if (!my_stricmp(arg, "-max"))
 		{
 			if ((stuff = next_arg(args, &args)))
+			{
+			    if (get_server(refnum))
 				get_server(refnum)->ison_max = atol(stuff);
+			}
 			else
 				say("Need numeric argument for -MAX argument.");
 		}
@@ -1346,8 +1358,12 @@ void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 			next = p;
 		while (*p && !isspace((unsigned char)*p))
 		    p++;
-		if (p - next < get_server(refnum)->ison_len)
+
+		/* You know, just in case something happened... */
+		if (get_server(refnum))
 		{
+		    if (p - next < get_server(refnum)->ison_len)
+		    {
 			/* can add this nick to the current list. */
 			if (!end)
 			{
@@ -1357,16 +1373,20 @@ void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 				if (new_i)
 					ison_queue_send(refnum);
 	
-				new_i = get_new_ison_entry(refnum, sendnext);
+				if (!(new_i = get_new_ison_entry(refnum, sendnext)))
+				{
+					privileged_yell("isonbase: Could not create a new ison entry for %d [%d]", refnum, sendnext);
+					continue;
+				}
 				new_i->line = line;
 				malloc_strcpy(&new_i->oncmd, on_cmd);
 				malloc_strcpy(&new_i->offcmd, offcmd);
 			}
 				
 			end = p;
-		}
-		else
-		{
+		    }
+		    else
+		    {
 			/* This nick is too long to add to the current list. */
 			if (end)
 			{
@@ -1382,6 +1402,7 @@ void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 				p = end;	
 				end = NULL;
 			}
+		    }
 		}
 		
 		/* Advance to next nick */
